<?php
namespace themeDevSocial\Apps\Counters;

if( ! defined( 'ABSPATH' )) die( 'Forbidden' );

use \themeDevSocial\Apps\Settings as Settings;
use \themeDevSocialNext\Providers\Abst\Util as Util;
use \themeDevSocialNext\Providers\Abst\Curl as Curl;

class Facebook extends Settings{
	
	 /**
     * Custom post type
     * Method Description: Set custom post type
     * @since 1.0.0
     * @access public
	 */
	private $api_data = ''; 
	
	private $api_key = ''; 
	
	private $api_secret = ''; 
	
	private $access_token = ''; 
	
	private $user_id = 'themedev2019'; 
	
	protected $apiBaseUrl = 'https://graph.facebook.com/v3.0/';

    protected $authorizeUrl = 'https://www.facebook.com/dialog/oauth';

    protected $accessTokenUrl = 'https://graph.facebook.com/v3.0/oauth/access_token';

    protected $profileUrlTemplate = 'https://www.facebook.com/%s';
	
    private $callback = '';

	private $token = '';
	
	private $token_data = '';
    
    private $curl;
	/**
     * Method: __construct
     * Method Description: load services
     * @since 1.0.0
     * @access private
	 */
	public function __construct($load = true){
		if($load){
			$this->api_data = parent::__get_api_data_counter('facebook');
			$this->api_key = isset($this->api_data['id']) ? $this->api_data['id'] : '';
            $this->api_secret = isset($this->api_data['secret']) ? $this->api_data['secret'] : '';
            $this->user_id = isset($this->api_data['user_id']) ? $this->api_data['user_id'] : $this->user_id;

			$this->token_data = isset($this->api_data['token']) ? $this->api_data['token'] : $this->token;

			$this->callback = get_site_url().'/wp-admin/admin.php?page=next-social-counter&tab=providers&type=facebook';
            
            $this->curl = new Curl();
        }
	}

    /**
     * Method: get_access_token
     * Method Description: Get Access Token
     * @since 1.0.0
     * @access public
	 */
    
     public function __get_accesstoken(){
		if(strlen( trim($this->token_data) ) > 5){
			$this->token = trim($this->token_data);
			return $this->token;
		}
		
		$tokenData   = get_transient( '_next_social_access_token' );
        $dataToken = isset($tokenData['facebook_token']) ? $tokenData['facebook_token'] : '';

		$get_transient_time   = get_transient( 'timeout__next_social_access_token' );		
        if( $get_transient_time < time() || empty(trim($dataToken))){
            $access_tok = parent::_get_social_services('social_token');
            if(isset($access_tok['data'])){
                set_transient( '_next_social_access_token', $access_tok['data'] , parent::$cache*60*60 );
            }
        }
        $tokenData   = get_transient( '_next_social_access_token' );
        $this->token = isset($tokenData['facebook_token']) ? $tokenData['facebook_token'] : '';
        return  $this->token;
     }

     /**
     * Method: __get_follower_data
     * Method Description: Get Followers Info
     * @since 1.0.0
     * @access public
	 */
    
    
	public function __get_follower_data(){
		if(strlen( trim($this->token_data) ) < 5){
			return $this->__like_counter();
		}
		
		if( strlen($this->token) > 5){	
			$header = [
				'Authorization' => sprintf('Bearer %s', $this->token),
				'Content-Type' => 'application/x-www-form-urlencoded;charset=UTF-8',
				'Accept' => 'application/json'
			];
			
			$body = [];
			try {
				//$url = $this->apiBaseUrl.'me/likes';
				$url = $this->apiBaseUrl.$this->user_id.'/likes';
				$reoponse = $this->curl->request( $url , 'GET', $body, $header);
				$reoponse = @json_decode($reoponse, true);
				
				if( isset($reoponse['likes']) && strlen($reoponse['likes']) > 0 ){
					
					return isset($reoponse['likes']) ? (int) $reoponse['likes'] : 0;
					
				}
			}
			catch (Exception $e) {
				return $this->__like_counter();
			}		
		}
		return $this->__like_counter();
	}

	private function __like_counter(){
		try {
			$counter = '';
			$url = 'https://www.facebook.com/plugins/likebox.php?href=https://facebook.com/'.$this->user_id.'&show_faces=true&header=false&stream=false&show_border=false&locale=en_US';
			$get_request = wp_remote_get($url, array( 'timeout' => 20 ) );
			$the_request = wp_remote_retrieve_body( $get_request );

			$pattern = '/_1drq[^>]+>(.*?)<\/a/s';
			preg_match( $pattern, $the_request, $matches );

			if ( ! empty( $matches[1] ) ) {
				$number  = strip_tags( $matches[1] );
				
				foreach ( str_split( $number ) as $char ) {
					if ( is_numeric( $char ) ){
						$counter .= $char;
					}
				}
			}
			return (int) $counter;
		}
		catch (Exception $e) {
			return 0;
		}
	}


	/**
     * Method: api_authorise
     * Method Description: Api Authentication
     * @since 1.0.0
     * @access private
	 */
	public function __api_authorise(){
		
		$params = [
			'response_type' => 'code',
			'client_id'     => $this->api_key,
			'scope'         => 'email, public_profile',
			'state'         => uniqid( '', true ),
			'redirect_uri'  => $this->callback,
		];
		
		$url = $this->authorizeUrl.'?' . http_build_query($params);
		Util::redirect($url);
	}
	/**
     * Method: __get_accesstoken_generate
     * Method Description: Generate Access Token
     * @since 1.0.0
     * @access private
	 */
	public function __get_accesstoken_generate($get_code = ''){	
		$credentials = $this->api_key . ':' . $this->api_secret;
		$toSend 	 = base64_encode($credentials);
		
		$header = [
				'Authorization' => 'bearer ' . $toSend,
				'Content-Type' 	=> 'application/x-www-form-urlencoded;charset=UTF-8',
				'Accept' => 'application/json'
			];
		$body = [ 'grant_type' => 'authorization_code', 'code' => $get_code, 'client_id' => $this->api_key, 'client_secret' => $this->api_secret, 'redirect_uri' => $this->callback ];
		
		
		$reoponse = $this->curl->request($this->accessTokenUrl, 'POST', $body, $header);
		$reoponse = @json_decode($reoponse, true);
		
		if( isset($reoponse['access_token']) ){
			return $reoponse['access_token'];
		}
		return $this->token_data;
	}
}