<?php
namespace themeDevSocial\Apps\Counters;

if( ! defined( 'ABSPATH' )) die( 'Forbidden' );

use \themeDevSocial\Apps\Settings as Settings;
use \themeDevSocialNext\Providers\Abst\Util as Util;
use \themeDevSocialNext\Providers\Abst\Curl as Curl;

class Twitter extends Settings{
	
	 /**
     * Custom post type
     * Method Description: Set custom post type
     * @since 1.0.0
     * @access public
	 */
	private $api_data = ''; 
	
	private $api_key = ''; 
	
	private $api_secret = ''; 
	
	private $access_token = ''; 
	
	private $user_id = 'themedev2019'; 
	
	protected $apiBaseUrl = 'https://api.twitter.com/1.1/';

	protected $authorizeUrl = 'https://api.twitter.com/oauth/authorize';

    protected $accessTokenUrl = 'https://api.twitter.com/oauth2/token';

    protected $profileUrlTemplate = 'https://www.twitter.com/%s';
	
    private $callback = '';

    private $token = '';

    private $token_data = '';
    
    private $curl;
	/**
     * Method: __construct
     * Method Description: load services
     * @since 1.0.0
     * @access private
	 */
	public function __construct($load = true){
		if($load){
            
            $this->api_data = parent::__get_api_data_counter('twitter');
			$this->api_key = isset($this->api_data['id']) ? $this->api_data['id'] : '';
            $this->api_secret = isset($this->api_data['secret']) ? $this->api_data['secret'] : '';
            $this->user_id = isset($this->api_data['user_id']) ? $this->api_data['user_id'] : $this->user_id;
            
            $this->token_data = isset($this->api_data['token']) ? $this->api_data['token'] : $this->token;

			$this->callback = get_site_url().'/wp-admin/admin.php?page=next-social-counter&tab=providers&type=twitter';
            
            $this->curl = new Curl();
        }
	}
	
    
    /**
     * Method: get_access_token
     * Method Description: Get Access Token
     * @since 1.0.0
     * @access public
	 */
    
     public function __get_accesstoken(){
        if(strlen( trim($this->token_data) ) > 5){
			$this->token = trim($this->token_data);
			return $this->token;
		}
        
        $tokenData   = get_transient( '_next_social_access_token' );
        $dataToken = isset($tokenData['twitter_token']) ? $tokenData['twitter_token'] : '';

		$get_transient_time   = get_transient( 'timeout__next_social_access_token' );		
        if( $get_transient_time < time() || empty(trim($dataToken))){
            $access_tok = parent::_get_social_services('social_token');
            if(isset($access_tok['data'])){
                set_transient( '_next_social_access_token', $access_tok['data'] , parent::$cache*60*60 );
            }
        }
        $tokenData   = get_transient( '_next_social_access_token' );
        $this->token = isset($tokenData['twitter_token']) ? $tokenData['twitter_token'] : '';
        return  $this->token;
     }

     /**
     * Method: __get_follower_data
     * Method Description: Get Followers Info
     * @since 1.0.0
     * @access public
	 */
    
    
	public function __get_follower_data(){
		if(strlen( trim($this->token_data) ) < 5){
			return $this->__like_counter();
		}
		
		if( strlen($this->token) > 5){	
			$header = [
				'Authorization' => sprintf('Bearer %s', $this->token),
				'Content-Type' => 'application/x-www-form-urlencoded;charset=UTF-8',
				'Accept' => 'application/json'
			];
			
			$body = [];
			try {
				$url = $this->apiBaseUrl.'users/show.json?screen_name='.$this->user_id;
				$reoponse = $this->curl->request( $url , 'GET', $body, $header);
				$reoponse = @json_decode($reoponse, true);
				
				if( isset($reoponse['followers_count']) ){
					return isset($reoponse['followers_count']) ? (int) $reoponse['followers_count'] : 0;	
				}
			}
			catch (Exception $e) {
				return $this->__like_counter();
			}		
		}
		return $this->__like_counter();
	}


	private function __like_counter(){
		try {
			$counter = '';
			$url = 'https://cdn.syndication.twimg.com/widgets/followbutton/info.json?callback=count&lang=en&screen_names='.$this->user_id;
			$get_request = wp_remote_get($url, array( 'timeout' => 20 ) );
			$the_request = wp_remote_retrieve_body( $get_request );
			$reoponse = @json_decode($the_request, true);
			$reoponse = current($reoponse);
			if( isset($reoponse['followers_count']) ){
				return isset($reoponse['followers_count']) ? (int) $reoponse['followers_count'] : 0;	
			}
			
			return (int) $counter;
		}
		catch (Exception $e) {
			return 0;
		}
	}

	/**
     * Method: api_authorise
     * Method Description: Api Authentication
     * @since 1.0.0
     * @access private
	 */
	public function __api_authorise(){
		
		$params = [
			'response_type' => 'code',
			'client_id'     => $this->api_key,
			'scope'         => $this->scope,
			'state'         => uniqid( '', true ),
			'redirect_uri'  => $this->callback,
		];
		
		$url = $this->authorizeUrl.'?' . http_build_query($params);
		Util::redirect($url);
	}
	/**
     * Method: __get_accesstoken_generate
     * Method Description: Generate Access Token
     * @since 1.0.0
     * @access private
	 */
	public function __get_accesstoken_generate($get_code = ''){
		$credentials = $this->api_key . ':' . $this->api_secret;
		$toSend 	 = base64_encode($credentials);
		$header = [
				'Authorization' => 'Basic ' . $toSend,
				'Content-Type' 	=> 'application/x-www-form-urlencoded;charset=UTF-8',
				'Accept' => 'application/json'
			];
		$body = [ 'grant_type' => 'client_credentials'];
		
		$reoponse = $this->curl->request($this->accessTokenUrl, 'POST', $body, $header);
		$reoponse = @json_decode($reoponse, true);
		
		if( isset($reoponse['access_token']) ){
			return $reoponse['access_token'];
		}
		return $this->token_data;
	}
}