<?php
namespace themeDevSocialNext\Providers;
 /**
 * Next Authentication PHP Script
 * Method Description: Here is 13 social provider for authentication.
 * @develop: themeDev 
 * @company URL: themedev.net 
 * @since 1.0.0
 * @access public
 */
use \themeDevSocialNext\Providers\Abst\Abstractnext as Abstractnext;
use \themeDevSocialNext\Providers\Abst\Util as Util;
use \themeDevSocialNext\Providers\Abst\Curl as Curl;

class Linkedin extends Abstractnext{
	
	 /**
     * Custom post type
     * Method Description: Set custom post type
     * @since 1.0.0
     * @access public
	 */
	protected $api_key = ''; 
	
	protected $api_secret = ''; 
	
	private $access_token = ''; 
	
	private $access_code = ''; 
	
	protected $apiBaseUrl = 'https://api.linkedin.com/v2/';

    protected $authorizeUrl = 'https://www.linkedin.com/oauth/v2/authorization';

    protected $accessTokenUrl = 'https://www.linkedin.com/oauth/v2/accessToken';

    protected $apiDocumentation = 'https://developer.linkedin.com/docs/oauth2';

    protected $profileUrlTemplate = 'https://www.linkedin.com/in/';
	
	protected $scope = 'r_emailaddress r_liteprofile w_member_social';
	
	protected $callback = '';
	
	protected $fields = '(id,firstName,lastName,emailAddress,publicProfileUrl,profilePicture(displayImage),num-connections)';
           
	
	private $nx_connected = false;
	
	private $profile_data = '';
	
	private $curl;
	/**
     * Method: __construct
     * Method Description: load services
     * @since 1.0.0
     * @access private
	 */
	public function __construct( $config ){
		if(is_array($config)){
			$this->api_key = isset($config['keys']['id']) ? $config['keys']['id'] : '';
			$this->api_secret = isset($config['keys']['secret']) ? $config['keys']['secret'] : '';
			$this->callback = isset($config['callback']) ? $config['callback'] : '';
			$this->curl = new Curl();
		}
	}
	/**
     * Method: api_authorise
     * Method Description: Api Authentication
     * @since 1.0.0
     * @access private
	 */
	public function nx_authorise(){
		
		if(isset($_GET['code'])){
			$this->get_access_token();
		}else if(isset($_GET['error'])){
			return $_GET['error'];
		}else{
			$params = [
				'response_type' => 'code',
				'client_id'     => $this->api_key,
				'scope'         => $this->scope,
				'state'         => uniqid( '', true ), // unique long string
				'redirect_uri'  => $this->callback,
			];
			
			$url = $this->authorizeUrl.'?' . http_build_query($params);
			Util::redirect($url);
		}
		
	}
	/**
     * Method: get_access_token
     * Method Description: Get Access Token
     * @since 1.0.0
     * @access private
	 */
	protected function get_access_token(){
		$this->access_code = isset($_GET['code']) ? $_GET['code'] : '';
		if(strlen($this->access_code) > 1){
			$credentials = $this->api_key . ':' . $this->api_secret;
			$toSend 	 = base64_encode($credentials);
			
			$header = [
					'Authorization' => 'Basic ' . $toSend,
					'Content-Type' 	=> 'application/x-www-form-urlencoded;charset=UTF-8'
				];
			$body = [ 'grant_type' => 'authorization_code', 'code' => $this->access_code, 'client_id' => $this->api_key, 'client_secret' => $this->api_secret, 'redirect_uri' => $this->callback ];
			
			$reoponse = $this->curl->request($this->accessTokenUrl, 'POST', $body, $header);
			$reoponse = @json_decode($reoponse, true);
			
			if( isset($reoponse['access_token']) ){
				$this->access_token = $reoponse['access_token'];
				$this->nx_connected = true;
				
				$this->get_api_data();
				
			} else{
				echo ' '. isset($reoponse['error_description']) ?  $reoponse['error_description'] : '';
			}
			
		}
	}
	/**
     * Method: get_api_data
     * Method Description: Get Api Data
     * @since 1.0.0
     * @access private
	 */
	protected function get_api_data(){
		if( strlen($this->access_token) > 5){	
			$header = [
				'Authorization' => sprintf('Bearer %s', $this->access_token),
				'Content-Type' => 'application/json',
				'x-li-format' => 'json',
				//'X-Restli-Protocol-Version' => '2.0.0'
			];
			
			$body = [
				'projection' => $this->fields
			];
			try {
				$url = $this->apiBaseUrl.'me';
				$reoponse = $this->curl->request( $url , 'GET', $body, $header);
				$reoponse = @json_decode($reoponse, true);
				
				if( isset($reoponse['id']) && strlen($reoponse['id']) > 0 ){
					
					$userProfile = new \stdClass();
					$userProfile->identifier = strlen($reoponse['id']) ? $reoponse['id'] : substr(md5(microtime()),rand(0,26),10);
					$userProfile->firstName = isset($reoponse['firstName']['localized']['en_US']) ? $reoponse['firstName']['localized']['en_US'] : '';
					$userProfile->lastName = isset($reoponse['firstName']['localized']['en_US']) ? $reoponse['firstName']['localized']['en_US'] : '';
					$userProfile->displayName = trim($userProfile->firstName . ' ' . $userProfile->lastName);
					$userProfile->email = $userProfile->identifier;
					$userProfile->description = '';
					$userProfile->emailVerified = true;
					$userProfile->photoURL = isset($reoponse['publicProfileUrl']) ? $reoponse['publicProfileUrl'] : '';
					$userProfile->webSiteURL = '';
					$userProfile->gender = '';
					$userProfile->language = '';
					$userProfile->profileURL = '';
					$userProfile->region = '';
					$userProfile->data['connections'] = isset($reoponse['numConnections']) ? $reoponse['numConnections'] : 0;
					
					$this->profile_data = $userProfile;
				}else{
					$this->profile_data = ' Sorry could\'t found profile information.';
				}
			}
			catch (Exception $e) {
				$this->profile_data = 'Sorry api error';
			}		
		}else{
			$this->profile_data = 'Invalid Accexx Token';
		}
		return $this->profile_data;
	}
	/**
     * Method: is_connected
     * Method Description: Check Connection
     * @since 1.0.0
     * @access private
	 */
	public function is_connected(){
		return $this->nx_connected;
	}
	/**
     * Method: getUserProfile
     * Method Description: Get User Profile Data
     * @since 1.0.0
     * @access private
	 */
	public function getUserProfile(){
		return $this->profile_data;
	}
	/**
     * Method: getAccessToken
     * Method Description: Get User Access Token
     * @since 1.0.0
     * @access private
	 */
	public function getAccessToken(){
		return $this->access_token;
	}
	/**
     * Method: getAccessCode
     * Method Description: Get User Access Code
     * @since 1.0.0
     * @access private
	 */
	public function getAccessCode(){
		return $this->access_code;
	}
	
	/**
     * Method: getUser
     * Method Description: Get User Data
     * @since 1.0.0
     * @access private
	 */
	public function getUser($access_token = ''){
		$this->access_token = $access_token;
		return $this->get_api_data();
	}
}

